@@ -42,6 +42,12 @@
 
 QT_BEGIN_NAMESPACE
 
+QQuickIconImagePrivate::~QQuickIconImagePrivate()
+{
+    qDeleteAll(icon.entries);
+    icon.entries.clear();
+}
+
 bool QQuickIconImagePrivate::updateDevicePixelRatio(qreal targetDevicePixelRatio)
 {
     if (isThemeIcon) {
@@ -132,6 +138,7 @@ void QQuickIconImage::setName(const QString &name)
     if (d->icon.iconName == name)
         return;
 
+    qDeleteAll(d->icon.entries);
     d->icon = QIconLoader::instance()->loadIcon(name);
     if (isComponentComplete())
         d->updateIcon();
