@@ -77,7 +77,7 @@ QT_BEGIN_NAMESPACE
     \list
     \li \l orientation
     \li \l position
-    \li \l size
+    \li \l {ScrollBar::} {size}
     \li \l active
     \endlist
 
@@ -797,6 +797,14 @@ void QQuickScrollBarAttachedPrivate::initHorizontal()
     if (parent && parent == flickable->parentItem())
         horizontal->stackAfter(flickable);
 
+    // If a scroll bar was previously hidden (due to e.g. setting a new contentItem
+    // on a ScrollView), we need to make sure that we un-hide it.
+    // We don't bother checking if the item is actually the old one, because
+    // if it's not, all of the things the function does (setting parent, visibility, etc.)
+    // should be no-ops anyway.
+    if (auto control = qobject_cast<QQuickControl*>(q_func()->parent()))
+        QQuickControlPrivate::unhideOldItem(control, horizontal);
+
     layoutHorizontal();
     horizontal->setSize(area->property("widthRatio").toReal());
     horizontal->setPosition(area->property("xPosition").toReal());
@@ -818,6 +826,9 @@ void QQuickScrollBarAttachedPrivate::initVertical()
     if (parent && parent == flickable->parentItem())
         vertical->stackAfter(flickable);
 
+    if (auto control = qobject_cast<QQuickControl*>(q_func()->parent()))
+        QQuickControlPrivate::unhideOldItem(control, vertical);
+
     layoutVertical();
     vertical->setSize(area->property("heightRatio").toReal());
     vertical->setPosition(area->property("yPosition").toReal());
@@ -827,6 +838,16 @@ void QQuickScrollBarAttachedPrivate::cleanupHorizontal()
 {
     Q_ASSERT(flickable && horizontal);
 
+    QQuickControlPrivate::hideOldItem(horizontal);
+    // ScrollBar.qml has a binding to visible and ScrollView.qml has a binding to parent.
+    // If we just set visible to false and parent to null, these bindings will overwrite
+    // them upon component completion as part of the binding evaluation.
+    // That's why we remove the binding completely.
+    const QQmlProperty visibleProperty(horizontal, QStringLiteral("visible"));
+    const QQmlProperty parentProperty(horizontal, QStringLiteral("parent"));
+    QQmlPropertyPrivate::removeBinding(visibleProperty);
+    QQmlPropertyPrivate::removeBinding(parentProperty);
+
     disconnect(flickable, &QQuickFlickable::movingHorizontallyChanged, this, &QQuickScrollBarAttachedPrivate::activateHorizontal);
 
     // TODO: export QQuickFlickableVisibleArea
@@ -839,6 +860,12 @@ void QQuickScrollBarAttachedPrivate::cleanupVertical()
 {
     Q_ASSERT(flickable && vertical);
 
+    QQuickControlPrivate::hideOldItem(vertical);
+    const QQmlProperty visibleProperty(vertical, QStringLiteral("visible"));
+    const QQmlProperty parentProperty(vertical, QStringLiteral("parent"));
+    QQmlPropertyPrivate::removeBinding(visibleProperty);
+    QQmlPropertyPrivate::removeBinding(parentProperty);
+
     disconnect(flickable, &QQuickFlickable::movingVerticallyChanged, this, &QQuickScrollBarAttachedPrivate::activateVertical);
 
     // TODO: export QQuickFlickableVisibleArea
@@ -869,6 +896,9 @@ class QQuickFriendlyFlickable : public QQuickFlickable
 
 void QQuickScrollBarAttachedPrivate::scrollHorizontal()
 {
+    if (!flickable)
+        return;
+
     QQuickFriendlyFlickable *f = reinterpret_cast<QQuickFriendlyFlickable *>(flickable);
 
     const qreal viewwidth = f->width();
@@ -881,6 +911,9 @@ void QQuickScrollBarAttachedPrivate::scrollHorizontal()
 
 void QQuickScrollBarAttachedPrivate::scrollVertical()
 {
+    if (!flickable)
+        return;
+
     QQuickFriendlyFlickable *f = reinterpret_cast<QQuickFriendlyFlickable *>(flickable);
 
     const qreal viewheight = f->height();
