@@ -55,7 +55,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype TextArea
     \inherits TextEdit
-    \instantiates QQuickTextArea
+//!     \instantiates QQuickTextArea
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-input
@@ -121,8 +121,8 @@ QT_BEGIN_NAMESPACE
     \qmlsignal QtQuick.Controls::TextArea::pressAndHold(MouseEvent event)
 
     This signal is emitted when there is a long press (the delay depends on the platform plugin).
-    The \l {MouseEvent}{event} parameter provides information about the press, including the x and y
-    position of the press, and which button is pressed.
+    The \a event parameter provides information about the press, including the x and y
+    coordinates of the press, and which button is pressed.
 
     \sa pressed, released
 */
@@ -132,8 +132,8 @@ QT_BEGIN_NAMESPACE
     \since QtQuick.Controls 2.1 (Qt 5.8)
 
     This signal is emitted when the text area is pressed by the user.
-    The \l {MouseEvent}{event} parameter provides information about the press,
-    including the x and y position of the press, and which button is pressed.
+    The \a event parameter provides information about the press,
+    including the x and y coordinates of the press, and which button is pressed.
 
     \sa released, pressAndHold
 */
@@ -143,8 +143,9 @@ QT_BEGIN_NAMESPACE
     \since QtQuick.Controls 2.1 (Qt 5.8)
 
     This signal is emitted when the text area is released by the user.
-    The \l {MouseEvent}{event} parameter provides information about the release,
-    including the x and y position of the press, and which button is pressed.
+    The \a event parameter provides information about the release,
+    including the x and y coordinates of the press, and which button
+    is pressed.
 
     \sa pressed, pressAndHold
 */
@@ -218,16 +219,25 @@ void QQuickTextAreaPrivate::resizeBackground()
 
     resizingBackground = true;
 
+    // When using the attached property TextArea.flickable, we reparent the background out
+    // of TextArea and into the Flickable since we don't want the background to move while
+    // flicking. This means that the size of the background should also follow the size of
+    // the Flickable rather than the size of the TextArea.
+    const auto flickable = qobject_cast<QQuickFlickable *>(background->parentItem());
+
     QQuickItemPrivate *p = QQuickItemPrivate::get(background);
     if (((!p->widthValid || !extra.isAllocated() || !extra->hasBackgroundWidth) && qFuzzyIsNull(background->x()))
             || (extra.isAllocated() && (extra->hasLeftInset || extra->hasRightInset))) {
+        const qreal bgWidth = flickable ? flickable->width() : width;
         background->setX(getLeftInset());
-        background->setWidth(width - getLeftInset() - getRightInset());
+        background->setWidth(bgWidth - getLeftInset() - getRightInset());
     }
+
     if (((!p->heightValid || !extra.isAllocated() || !extra->hasBackgroundHeight) && qFuzzyIsNull(background->y()))
             || (extra.isAllocated() && (extra->hasTopInset || extra->hasBottomInset))) {
+        const qreal bgHeight = flickable ? flickable->height() : height;
         background->setY(getTopInset());
-        background->setHeight(height - getTopInset() - getBottomInset());
+        background->setHeight(bgHeight - getTopInset() - getBottomInset());
     }
 
     resizingBackground = false;
@@ -253,7 +263,10 @@ void QQuickTextAreaPrivate::inheritFont(const QFont &font)
     parentFont.resolve(extra.isAllocated() ? extra->requestedFont.resolve() | font.resolve() : font.resolve());
 
     const QFont defaultFont = QQuickTheme::font(QQuickTheme::TextArea);
-    const QFont resolvedFont = parentFont.resolve(defaultFont);
+    QFont resolvedFont = parentFont.resolve(defaultFont);
+    // See comment in QQuickControlPrivate::inheritFont
+    if (defaultFont.families().isEmpty())
+        resolvedFont.setFamilies(QStringList());
 
     setFont_helper(resolvedFont);
 }
@@ -345,6 +358,7 @@ void QQuickTextAreaPrivate::attachFlickable(QQuickFlickable *item)
     QObject::connect(flickable, &QQuickFlickable::contentYChanged, q, &QQuickItem::update);
 
     QQuickItemPrivate::get(flickable)->updateOrAddGeometryChangeListener(this, QQuickGeometryChange::Size);
+    QQuickItemPrivate::get(flickable)->addItemChangeListener(this, QQuickItemPrivate::Destroyed);
     QObjectPrivate::connect(flickable, &QQuickFlickable::contentWidthChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
     QObjectPrivate::connect(flickable, &QQuickFlickable::contentHeightChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
 
@@ -365,10 +379,13 @@ void QQuickTextAreaPrivate::detachFlickable()
     QObject::disconnect(flickable, &QQuickFlickable::contentYChanged, q, &QQuickItem::update);
 
     QQuickItemPrivate::get(flickable)->updateOrRemoveGeometryChangeListener(this, QQuickGeometryChange::Nothing);
+    QQuickItemPrivate::get(flickable)->removeItemChangeListener(this, QQuickItemPrivate::Destroyed);
     QObjectPrivate::disconnect(flickable, &QQuickFlickable::contentWidthChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
     QObjectPrivate::disconnect(flickable, &QQuickFlickable::contentHeightChanged, this, &QQuickTextAreaPrivate::resizeFlickableControl);
 
     flickable = nullptr;
+
+    resizeBackground();
 }
 
 void QQuickTextAreaPrivate::ensureCursorVisible()
@@ -433,11 +450,21 @@ void QQuickTextAreaPrivate::resizeFlickableContent()
 
 void QQuickTextAreaPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &diff)
 {
-    Q_UNUSED(item);
-    Q_UNUSED(change);
     Q_UNUSED(diff);
+    if (!resizingBackground && item == background) {
+        QQuickItemPrivate *p = QQuickItemPrivate::get(item);
+        // Only set hasBackgroundWidth/Height if it was a width/height change,
+        // otherwise we're prevented from setting a width/height in the future.
+        if (change.widthChange())
+            extra.value().hasBackgroundWidth = p->widthValid;
+        if (change.heightChange())
+            extra.value().hasBackgroundHeight = p->heightValid;
+    }
 
-    resizeFlickableControl();
+    if (flickable)
+        resizeFlickableControl();
+    else
+        resizeBackground();
 }
 
 qreal QQuickTextAreaPrivate::getImplicitWidth() const
@@ -485,7 +512,7 @@ void QQuickTextAreaPrivate::accessibilityActiveChanged(bool active)
     Q_Q(QQuickTextArea);
     QQuickAccessibleAttached *accessibleAttached = qobject_cast<QQuickAccessibleAttached *>(qmlAttachedPropertiesObject<QQuickAccessibleAttached>(q, true));
     Q_ASSERT(accessibleAttached);
-    accessibleAttached->setRole(accessibleRole());
+    accessibleAttached->setRole(effectiveAccessibleRole());
     accessibleAttached->set_readOnly(q->isReadOnly());
     accessibleAttached->setDescription(placeholder);
 }
@@ -512,11 +539,8 @@ void QQuickTextAreaPrivate::executeBackground(bool complete)
 
     if (!background || complete)
         quickBeginDeferred(q, backgroundName(), background);
-    if (complete) {
+    if (complete)
         quickCompleteDeferred(q, backgroundName(), background);
-        if (background)
-            QQuickControlPrivate::addImplicitSizeListener(background, this, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
-    }
 }
 
 void QQuickTextAreaPrivate::itemImplicitWidthChanged(QQuickItem *item)
@@ -540,6 +564,8 @@ void QQuickTextAreaPrivate::itemDestroyed(QQuickItem *item)
         background = nullptr;
         emit q->implicitBackgroundWidthChanged();
         emit q->implicitBackgroundHeightChanged();
+    } else if (item == flickable) {
+        detachFlickable();
     }
 }
 
@@ -621,25 +647,24 @@ void QQuickTextArea::setBackground(QQuickItem *background)
     }
 
     QQuickControlPrivate::removeImplicitSizeListener(d->background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
-    delete d->background;
+    QQuickControlPrivate::hideOldItem(d->background);
     d->background = background;
 
     if (background) {
+        QQuickItemPrivate *p = QQuickItemPrivate::get(background);
+        if (p->widthValid || p->heightValid) {
+            d->extra.value().hasBackgroundWidth = p->widthValid;
+            d->extra.value().hasBackgroundHeight = p->heightValid;
+        }
         if (d->flickable)
             background->setParentItem(d->flickable);
         else
             background->setParentItem(this);
         if (qFuzzyIsNull(background->z()))
             background->setZ(-1);
-        QQuickItemPrivate *p = QQuickItemPrivate::get(background);
-        if (p->widthValid || p->heightValid) {
-            d->extra.value().hasBackgroundWidth = p->widthValid;
-            d->extra.value().hasBackgroundHeight = p->heightValid;
-        }
-        if (isComponentComplete()) {
+        if (isComponentComplete())
             d->resizeBackground();
-            QQuickControlPrivate::addImplicitSizeListener(background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
-        }
+        QQuickControlPrivate::addImplicitSizeListener(background, d, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
     }
 
     if (!qFuzzyCompare(oldImplicitBackgroundWidth, implicitBackgroundWidth()))
@@ -1061,7 +1086,11 @@ QSGNode *QQuickTextArea::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
     if (d->flickable)
         clipper = d->flickable;
 
-    const QRectF cr = clipper->clipRect().adjusted(leftPadding(), topPadding(), -rightPadding(), -bottomPadding());
+    const QRectF cr = clipper->clipRect().adjusted(
+            leftPadding(), topPadding(),
+            (!d->cursorItem && effectiveHAlign() == HAlignment::AlignRight ? 1 : 0) - rightPadding(),
+            -bottomPadding());
+
     clipNode->setRect(!d->flickable ? cr : cr.translated(d->flickable->contentX(), d->flickable->contentY()));
     clipNode->update();
 
