@@ -101,11 +101,13 @@ private slots:
     void menuItemWidthAfterImplicitWidthChanged_data();
     void menuItemWidthAfterImplicitWidthChanged();
     void menuItemWidthAfterRetranslate();
+    void giveMenuItemFocusOnButtonPress();
 };
 
 void tst_QQuickMenu::defaults()
 {
     QQuickApplicationHelper helper(this, QLatin1String("applicationwindow.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickMenu *emptyMenu = helper.appWindow->property("emptyMenu").value<QQuickMenu*>();
     QCOMPARE(emptyMenu->isVisible(), false);
@@ -117,6 +119,7 @@ void tst_QQuickMenu::defaults()
 void tst_QQuickMenu::count()
 {
     QQuickApplicationHelper helper(this, QLatin1String("applicationwindow.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickMenu *menu = helper.window->property("emptyMenu").value<QQuickMenu*>();
     QVERIFY(menu);
@@ -149,6 +152,7 @@ void tst_QQuickMenu::mouse()
         QSKIP("Mouse hovering not functional on offscreen/minimal platforms");
 
     QQuickApplicationHelper helper(this, QLatin1String("applicationwindow.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
@@ -244,6 +248,7 @@ void tst_QQuickMenu::mouse()
 void tst_QQuickMenu::pressAndHold()
 {
     QQuickApplicationHelper helper(this, QLatin1String("pressAndHold.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickWindow *window = helper.window;
     window->show();
@@ -268,6 +273,7 @@ void tst_QQuickMenu::contextMenuKeyboard()
         QSKIP("This platform only allows tab focus for text controls");
 
     QQuickApplicationHelper helper(this, QLatin1String("applicationwindow.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
@@ -451,6 +457,7 @@ void tst_QQuickMenu::disabledMenuItemKeyNavigation()
         QSKIP("This platform only allows tab focus for text controls");
 
     QQuickApplicationHelper helper(this, QLatin1String("disabledMenuItemKeyNavigation.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
@@ -515,6 +522,7 @@ void tst_QQuickMenu::mnemonics()
 #endif
 
     QQuickApplicationHelper helper(this, QLatin1String("mnemonics.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickWindow *window = helper.window;
     window->show();
@@ -562,6 +570,7 @@ void tst_QQuickMenu::menuButton()
         QSKIP("This platform only allows tab focus for text controls");
 
     QQuickApplicationHelper helper(this, QLatin1String("applicationwindow.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
 
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
@@ -588,6 +597,7 @@ void tst_QQuickMenu::menuButton()
 void tst_QQuickMenu::addItem()
 {
     QQuickApplicationHelper helper(this, QLatin1String("addItem.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -609,6 +619,7 @@ void tst_QQuickMenu::addItem()
 void tst_QQuickMenu::menuSeparator()
 {
     QQuickApplicationHelper helper(this, QLatin1String("menuSeparator.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -686,6 +697,7 @@ void tst_QQuickMenu::menuSeparator()
 void tst_QQuickMenu::repeater()
 {
     QQuickApplicationHelper helper(this, QLatin1String("repeater.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -730,6 +742,7 @@ void tst_QQuickMenu::repeater()
 void tst_QQuickMenu::order()
 {
     QQuickApplicationHelper helper(this, QLatin1String("order.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -751,6 +764,7 @@ void tst_QQuickMenu::order()
 void tst_QQuickMenu::popup()
 {
     QQuickApplicationHelper helper(this, QLatin1String("popup.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -907,6 +921,7 @@ void tst_QQuickMenu::popup()
 void tst_QQuickMenu::actions()
 {
     QQuickApplicationHelper helper(this, QLatin1String("actions.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -975,6 +990,7 @@ void tst_QQuickMenu::actions()
 void tst_QQuickMenu::removeTakeItem()
 {
     QQuickApplicationHelper helper(this, QLatin1String("removeTakeItem.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1038,6 +1054,7 @@ void tst_QQuickMenu::subMenuMouse()
     QFETCH(bool, cascade);
 
     QQuickApplicationHelper helper(this, QLatin1String("subMenus.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -1144,6 +1161,7 @@ void tst_QQuickMenu::subMenuDisabledMouse()
     QFETCH(bool, cascade);
 
     QQuickApplicationHelper helper(this, QLatin1String("subMenuDisabled.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -1206,6 +1224,7 @@ void tst_QQuickMenu::subMenuKeyboard()
     QFETCH(bool, mirrored);
 
     QQuickApplicationHelper helper(this, QLatin1String("subMenus.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -1328,6 +1347,7 @@ void tst_QQuickMenu::subMenuDisabledKeyboard()
     QFETCH(bool, mirrored);
 
     QQuickApplicationHelper helper(this, QLatin1String("subMenuDisabled.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     centerOnScreen(window);
     moveMouseAway(window);
@@ -1378,6 +1398,12 @@ void tst_QQuickMenu::subMenuDisabledKeyboard()
     QVERIFY(!subMenu->isVisible());
 }
 
+/*
+    QCOMPARE() compares doubles with 1-in-1e12 precision, which is too fine for these tests.
+    Casting to floats, compared with 1-in-1e5 precision, gives more robust results.
+*/
+#define FLOAT_EQ(u, v) QCOMPARE(float(u), float(v))
+
 void tst_QQuickMenu::subMenuPosition_data()
 {
     QTest::addColumn<bool>("cascade");
@@ -1404,6 +1430,7 @@ void tst_QQuickMenu::subMenuPosition()
     QFETCH(qreal, overlap);
 
     QQuickApplicationHelper helper(this, QLatin1String("subMenus.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
 
     // Ensure that the default size of the window fits three menus side by side.
@@ -1477,19 +1504,20 @@ void tst_QQuickMenu::subMenuPosition()
     if (cascade) {
         QCOMPARE(subMenu1->parentItem(), subMenu1Item);
         // vertically aligned to the parent menu item
-        QCOMPARE(subMenu1->popupItem()->y(), mainMenu->popupItem()->y() + subMenu1Item->y());
+        // We cast to float here because we want to use its larger tolerance for equality (because it has less precision than double).
+        FLOAT_EQ(subMenu1->popupItem()->y(), mainMenu->popupItem()->y() + subMenu1Item->y());
         if (mirrored) {
             // on the left of the parent menu
-            QCOMPARE(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() - subMenu1->width() + overlap);
+            FLOAT_EQ(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() - subMenu1->width() + overlap);
         } else {
             // on the right of the parent menu
-            QCOMPARE(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() + mainMenu->width() - overlap);
+            FLOAT_EQ(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() + mainMenu->width() - overlap);
         }
     } else {
         QCOMPARE(subMenu1->parentItem(), mainMenu->parentItem());
         // centered over the parent menu
-        QCOMPARE(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() + (mainMenu->width() - subMenu1->width()) / 2);
-        QCOMPARE(subMenu1->popupItem()->y(), mainMenu->popupItem()->y() + (mainMenu->height() - subMenu1->height()) / 2);
+        FLOAT_EQ(subMenu1->popupItem()->x(), mainMenu->popupItem()->x() + (mainMenu->width() - subMenu1->width()) / 2);
+        FLOAT_EQ(subMenu1->popupItem()->y(), mainMenu->popupItem()->y() + (mainMenu->height() - subMenu1->height()) / 2);
     }
 
     // open the sub-sub-menu (can flip)
@@ -1506,25 +1534,28 @@ void tst_QQuickMenu::subMenuPosition()
     if (cascade) {
         QCOMPARE(subSubMenu1->parentItem(), subSubMenu1Item);
         // vertically aligned to the parent menu item
-        QCOMPARE(subSubMenu1->popupItem()->y(), subMenu1->popupItem()->y() + subSubMenu1Item->y());
+        FLOAT_EQ(subSubMenu1->popupItem()->y(), subMenu1->popupItem()->y() + subSubMenu1Item->y());
         if (mirrored != flip) {
             // on the left of the parent menu
-            QCOMPARE(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() - subSubMenu1->width() + overlap);
+            FLOAT_EQ(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() - subSubMenu1->width() + overlap);
         } else {
             // on the right of the parent menu
-            QCOMPARE(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() + subMenu1->width() - overlap);
+            FLOAT_EQ(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() + subMenu1->width() - overlap);
         }
     } else {
         QCOMPARE(subSubMenu1->parentItem(), subMenu1->parentItem());
         // centered over the parent menu
-        QCOMPARE(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() + (subMenu1->width() - subSubMenu1->width()) / 2);
-        QCOMPARE(subSubMenu1->popupItem()->y(), subMenu1->popupItem()->y() + (subMenu1->height() - subSubMenu1->height()) / 2);
+        FLOAT_EQ(subSubMenu1->popupItem()->x(), subMenu1->popupItem()->x() + (subMenu1->width() - subSubMenu1->width()) / 2);
+        FLOAT_EQ(subSubMenu1->popupItem()->y(), subMenu1->popupItem()->y() + (subMenu1->height() - subSubMenu1->height()) / 2);
     }
 }
 
+#undef FLOAT_EQ
+
 void tst_QQuickMenu::addRemoveSubMenus()
 {
     QQuickApplicationHelper helper(this, QLatin1String("subMenus.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1588,6 +1619,7 @@ void tst_QQuickMenu::scrollable_data()
 
     QTest::addRow("Window") << QString::fromLatin1("windowScrollable.qml");
     QTest::addRow("ApplicationWindow") << QString::fromLatin1("applicationWindowScrollable.qml");
+    QTest::addRow("WithPadding") << QString::fromLatin1("scrollableWithPadding.qml");
 }
 
 void tst_QQuickMenu::scrollable()
@@ -1595,6 +1627,7 @@ void tst_QQuickMenu::scrollable()
     QFETCH(QString, qmlFilePath);
 
     QQuickApplicationHelper helper(this, qmlFilePath);
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1634,6 +1667,7 @@ void tst_QQuickMenu::disableWhenTriggered()
     QFETCH(int, subMenuItemIndex);
 
     QQuickApplicationHelper helper(this, QLatin1String("disableWhenTriggered.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickWindow *window = helper.window;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1689,6 +1723,7 @@ void tst_QQuickMenu::menuItemWidth()
     QFETCH(bool, mirrored);
 
     QQuickApplicationHelper helper(this, QLatin1String("menuItemWidths.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1717,6 +1752,7 @@ void tst_QQuickMenu::menuItemWidthAfterMenuWidthChanged()
     QFETCH(bool, mirrored);
 
     QQuickApplicationHelper helper(this, QLatin1String("menuItemWidths.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1761,6 +1797,7 @@ void tst_QQuickMenu::menuItemWidthAfterImplicitWidthChanged()
     QFETCH(bool, mirrored);
 
     QQuickApplicationHelper helper(this, QLatin1String("menuItemWidths.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1791,6 +1828,7 @@ void tst_QQuickMenu::menuItemWidthAfterImplicitWidthChanged()
 void tst_QQuickMenu::menuItemWidthAfterRetranslate()
 {
     QQuickApplicationHelper helper(this, QLatin1String("menuItemWidths.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowActive(window));
@@ -1819,6 +1857,32 @@ void tst_QQuickMenu::menuItemWidthAfterRetranslate()
     }
 }
 
+void tst_QQuickMenu::giveMenuItemFocusOnButtonPress()
+{
+    QQuickApplicationHelper helper(this, QLatin1String("giveMenuItemFocusOnButtonPress.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickApplicationWindow *window = helper.appWindow;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    // Press enter on the button to open the menu.
+    QQuickButton *menuButton = window->property("menuButton").value<QQuickButton*>();
+    QVERIFY(menuButton);
+    menuButton->forceActiveFocus();
+    QVERIFY(menuButton->hasActiveFocus());
+
+    QSignalSpy clickedSpy(window, SIGNAL(menuButtonClicked()));
+    QVERIFY(clickedSpy.isValid());
+
+    QTest::keyClick(window, Qt::Key_Return);
+    QCOMPARE(clickedSpy.count(), 1);
+
+    // The menu should still be open.
+    QQuickMenu *menu = window->property("menu").value<QQuickMenu*>();
+    QVERIFY(menu);
+    QTRY_VERIFY(menu->isOpened());
+}
+
 QTEST_QUICKCONTROLS_MAIN(tst_QQuickMenu)
 
 #include "tst_qquickmenu.moc"
