@@ -73,12 +73,15 @@ private slots:
     void activeFocusOnClose2();
     void activeFocusOnClose3();
     void activeFocusOnClosingSeveralPopups();
+    void activeFocusAfterExit();
+    void activeFocusOnDelayedEnter();
     void hover_data();
     void hover();
     void wheel_data();
     void wheel();
     void parentDestroyed();
     void nested();
+    void modelessOnModalOnModeless();
     void grabber();
     void cursorShape();
     void componentComplete();
@@ -96,6 +99,7 @@ private slots:
     void tabFence();
     void invisibleToolTipOpen();
     void centerInOverlayWithinStackViewItem();
+    void destroyDuringExitTransition();
 };
 
 void tst_QQuickPopup::initTestCase()
@@ -727,6 +731,79 @@ void tst_QQuickPopup::activeFocusOnClosingSeveralPopups()
     QTRY_COMPARE(button->hasActiveFocus(), true);
 }
 
+void tst_QQuickPopup::activeFocusAfterExit()
+{
+    // Test that after closing a popup the highest one in z-order receives it instead.
+    QQuickApplicationHelper helper(this, QStringLiteral("activeFocusAfterExit.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickApplicationWindow *window = helper.appWindow;
+    window->show();
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QQuickPopup *popup1 = window->property("popup1").value<QQuickPopup*>();
+    QVERIFY(popup1);
+
+    QQuickPopup *popup2 = window->property("popup2").value<QQuickPopup*>();
+    QVERIFY(popup2);
+    QSignalSpy closedSpy2(popup2, SIGNAL(closed()));
+    QVERIFY(closedSpy2.isValid());
+
+    QQuickPopup *popup3 = window->property("popup3").value<QQuickPopup*>();
+    QVERIFY(popup3);
+    QSignalSpy closedSpy3(popup3, SIGNAL(closed()));
+    QVERIFY(closedSpy3.isValid());
+
+    popup1->open();
+    QVERIFY(popup1->isVisible());
+    QTRY_VERIFY(popup1->hasActiveFocus());
+
+    popup2->open();
+    QVERIFY(popup2->isVisible());
+    QTRY_VERIFY(!popup2->hasActiveFocus());
+
+    popup3->open();
+    QVERIFY(popup3->isVisible());
+    QTRY_VERIFY(popup3->hasActiveFocus());
+
+    popup3->close();
+    closedSpy3.wait();
+    QVERIFY(!popup3->isVisible());
+    QTRY_VERIFY(!popup3->hasActiveFocus());
+    QTRY_VERIFY(!popup2->hasActiveFocus());
+    QTRY_VERIFY(popup1->hasActiveFocus());
+
+    popup2->close();
+    closedSpy2.wait();
+    QVERIFY(!popup2->isVisible());
+    QTRY_VERIFY(!popup2->hasActiveFocus());
+    QTRY_VERIFY(popup1->hasActiveFocus());
+}
+
+void tst_QQuickPopup::activeFocusOnDelayedEnter()
+{
+    // Test that after opening two popups, first of which has an animation, does not cause
+    // the first one to receive focus after the animation stops.
+    QQuickApplicationHelper helper(this, QStringLiteral("activeFocusOnDelayedEnter.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickApplicationWindow *window = helper.appWindow;
+    window->show();
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QQuickPopup *popup1 = window->property("popup1").value<QQuickPopup*>();
+    QVERIFY(popup1);
+    QSignalSpy openedSpy(popup1, SIGNAL(opened()));
+
+    QQuickPopup *popup2 = window->property("popup2").value<QQuickPopup*>();
+    QVERIFY(popup2);
+
+    popup1->open();
+    popup2->open();
+    openedSpy.wait();
+    QTRY_VERIFY(popup2->hasActiveFocus());
+}
+
 void tst_QQuickPopup::hover_data()
 {
     QTest::addColumn<QString>("source");
@@ -914,6 +991,46 @@ void tst_QQuickPopup::nested()
     QCOMPARE(modalPopup->isVisible(), true);
 }
 
+void tst_QQuickPopup::modelessOnModalOnModeless()
+{
+    QQuickApplicationHelper helper(this, QStringLiteral("modelessOnModalOnModeless.qml"));
+    QVERIFY2(helper.ready, helper.failureMessage());
+    QQuickWindow *window = helper.window;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+
+    QQuickPopup *modelessPopup = window->property("modelessPopup").value<QQuickPopup *>();
+    QVERIFY(modelessPopup);
+
+    QQuickButton *button = window->property("button").value<QQuickButton *>();
+    QVERIFY(button);
+    QQuickPopup *modalPopup = window->property("modalPopup").value<QQuickPopup *>();
+    QVERIFY(modalPopup);
+    QQuickPopup *tooltip = window->property("tooltip").value<QQuickPopup *>();
+    QVERIFY(modalPopup);
+
+    modelessPopup->open();
+    QCOMPARE(modelessPopup->isVisible(), true);
+    QTRY_COMPARE(modelessPopup->isOpened(), true);
+    const auto buttonPoint = button->mapToScene(button->boundingRect().center()).toPoint();
+    // click into the button, should not be blocked
+    QTest::mouseClick(window, Qt::LeftButton, Qt::NoModifier, buttonPoint);
+    QVERIFY(button->isChecked());
+    modalPopup->open();
+    QCOMPARE(modalPopup->isVisible(), true);
+    QTRY_COMPARE(modalPopup->isOpened(), true);
+    // click into the button, should be blocked
+    QTest::mouseClick(window, Qt::LeftButton, Qt::NoModifier, buttonPoint);
+    QVERIFY(button->isChecked());
+
+    tooltip->setVisible(true);
+    QCOMPARE(tooltip->isVisible(), true);
+    QTRY_COMPARE(tooltip->isOpened(), true);
+    // click into the button, should be blocked
+    QTest::mouseClick(window, Qt::LeftButton, Qt::NoModifier, buttonPoint);
+    QVERIFY(button->isChecked());
+}
+
 // QTBUG-56697
 void tst_QQuickPopup::grabber()
 {
@@ -1429,6 +1546,35 @@ void tst_QQuickPopup::centerInOverlayWithinStackViewItem()
     // Shouldn't crash on exit.
 }
 
+void tst_QQuickPopup::destroyDuringExitTransition()
+{
+    QQuickApplicationHelper helper(this, "destroyDuringExitTransition.qml");
+    QVERIFY2(helper.ready, helper.failureMessage());
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QPointer<QQuickPopup> dialog2 = window->property("dialog2").value<QQuickPopup*>();
+    QVERIFY(dialog2);
+    QTRY_COMPARE(dialog2->isVisible(), true);
+
+    // Close the second dialog, destroying it before its exit transition can finish.
+    QTest::keyClick(window, Qt::Key_Escape);
+    QTRY_VERIFY(!dialog2);
+
+    // Events should go through to the dialog underneath.
+    QQuickPopup *dialog1 = window->property("dialog1").value<QQuickPopup*>();
+    QVERIFY(dialog1);
+    QQuickButton *button = dialog1->property("button").value<QQuickButton*>();
+    QVERIFY(button);
+    const auto buttonClickPos = button->mapToScene(QPointF(button->width() / 2, button->height() / 2)).toPoint();
+    QTest::mousePress(window, Qt::LeftButton, Qt::NoModifier, buttonClickPos);
+    QVERIFY(button->isDown());
+    QTest::mouseRelease(window, Qt::LeftButton, Qt::NoModifier, buttonClickPos);
+    QVERIFY(!button->isDown());
+}
+
 QTEST_QUICKCONTROLS_MAIN(tst_QQuickPopup)
 
 #include "tst_qquickpopup.moc"
